@@ -10,6 +10,7 @@ from .tools import (
     check_file_exist,
     upload_to_s3,
     delete_file,
+    has_items,
 )
 
 
@@ -52,24 +53,32 @@ class BaseDataManager(object):
     def serialize_data(self, data: Any, fields: list) -> Union[Generator, list, dict]:
         raise NotImplementedError('serialize_data must be implemented.')
 
-    def _generate_file(self, skip: int, s3_filename: str, fields: list) -> tuple:
-        if check_file_exist(s3_filename):
-            skip += self.default_limit
-            return get_file_link(s3_filename)
+    def _generate_file(self, s3_filename: str, fields: list, skip: int):
         with open(s3_filename, 'w') as f:
             writer = csv.DictWriter(f, delimiter='\t', fieldnames=fields)
             writer.writeheader()
             last_id = None
             while True:
-                qs = self.get_data(skip=skip, last_id=last_id)
-                if not qs:
+                result = self.get_data(skip=skip, last_id=last_id)
+                exist, data = has_items(result)
+                if not exist:
                     break
                 if self.with_last_id:
-                    last_id = getattr(qs[-1], self.last_id_field)
-                data = self.serialize_data(qs, fields)
+                    last_id = (
+                        getattr(data[-1], self.last_id_field)
+                        if isinstance(data, list)
+                        else getattr(list(data)[-1], self.last_id_field)
+                    )
+                data = self.serialize_data(data, fields)
                 for row in data:
                     writer.writerow(row)
                 skip += self.query_limit
+
+    def _generate_file_link(self, skip: int, s3_filename: str, fields: list) -> tuple:
+        if check_file_exist(s3_filename):
+            skip += self.default_limit
+            return get_file_link(s3_filename)
+        self._generate_file(s3_filename, fields, skip)
         s3_file_data = upload_to_s3(s3_filename)
         return skip, get_file_link(s3_file_data['s3_filename'])
 
@@ -79,7 +88,7 @@ class BaseDataManager(object):
         count = self.get_count()
         links = []
         if self.default_limit >= count:
-            skip, link = self._generate_file(skip, f'{filename}.tsv', fields)
+            skip, link = self._generate_file_link(skip, f'{filename}.tsv', fields)
             links.append(link)
             delete_file(f'{filename}.tsv')
         else:
@@ -88,7 +97,9 @@ class BaseDataManager(object):
                     part += 1
                     part_filename = f'{filename}_{part}'
                     s3_filename = f"{part_filename}.tsv"
-                    skip, link = self._generate_file(skip, f'{filename}.tsv', fields)
+                    skip, link = self._generate_file_link(
+                        skip, f'{filename}.tsv', fields
+                    )
                     links.append(link)
                     delete_file(s3_filename)
         return {'links': links}
